﻿# -*- coding: utf-8 -*-
"""
制御文字が含まれていた場合にエラーとする
"""

import cse
import cseMessageBox
import re

# \t (\x09), \n (\x0a), \r (\x0d) を除く ASCII制御文字
rePattern = re.compile(r"[\x00-\x08\x0b\x0c\x0e-\x1f\x7f]")


# 制御文字が含まれているかどうかを判定
def hasCtrlChars(cont):
  if str(cont.type) == "text" or str(cont.type) == "multitext":
    if rePattern.search(cont.Value) is not None:
      return True
    else:
      return False
  elif str(cont.type) == "list":
    strt = ''.join([listItem for listItem in cont.Value])
    if rePattern.search(strt) is not None:
      return True
    else:
      return False


# 制御文字を取り除く
def removeCtrlChars(cont):
  if str(cont.type) == "text" or str(cont.type) == "multitext":
    cont.Value = rePattern.sub('', cont.Value)
  elif str(cont.type) == "list":
    for i in range(len(cont.Value)):
      cont.Value[i] = rePattern.sub('', cont.Value[i])


# 出力シートチェックメソッド
def CheckOutSheet(s, e):
  # プラグイン実行イベントデータ
  v = e.Value
  
  # エラーの種類
  errtype = "PlgErr_ChkCtrlChar_ContainsCtrlChar"
  
  # 親側、子側実装要素でループ
  for sti in v.sheets:
    if sti.IsShown: # 見えるチェックシートのみ
      for contset in cse.getcontssets(sti.Sheet, False):
        for flattenCont in contset:
          if hasCtrlChars(flattenCont):
            # 制御文字が含まれているものをエラーとする
            v.errors.append({'type': errtype, 'path': cse.deparsepath(flattenCont)})


# 入力シートチェックメソッド
def CheckImportedSheet(s, e):
  # プラグイン実行イベントデータ
  v = e.Value
  
  # エラー文字列のベース
  errMsgKey = "PlgMsg_ChkCtrlChar_RemoveCtrlChar"
  errMsgBase = cse.getapplang(errMsgKey)
  if not errMsgBase:
    # 空文字だったらエラー
    raise ('Lang key "%s" is not found.' % errMsgKey)

  # 制御文字が混じっている要素をリスト化
  havingCtrlCharCells = [flattenCont for sti in v.sheets for contset in cse.getcontssets(sti.Sheet, False) for flattenCont in contset if hasCtrlChars(flattenCont)]
  
  # リストが存在した場合、ダイアログで「はい」を選択したら、全ての制御文字を削除する
  if len(havingCtrlCharCells) > 0:
    # 制御文字を持つセルの先頭の 32 要素でエラーメッセージを作成
    errtargets = "\n".join("  " + cse.getcellpathtitle(str(cont.Path)) for cont in havingCtrlCharCells[:32])
    errmsg = errMsgBase % errtargets
    
    if len(havingCtrlCharCells) > 32:
      # 省略した数の表示
      errmsg += "\n  and more... (%d)" % (len(havingCtrlCharCells) - 32)
    
    if cseMessageBox.askyesno("", errmsg):
      for cont in havingCtrlCharCells:
        removeCtrlChars(cont)


cse.addhandler("RequestCheckOutSheet", CheckOutSheet)
cse.addhandler("AfterFileImported", CheckImportedSheet)
