﻿# -*- coding: utf-8 -*-
"""
エラーリストを HTML に埋め込みます
"""

import cse
import itertools

# Html ソースの設定
htmlStyleNode = '''<style type="text/css">
	table.errorlistout	{ margin-bottom: 25pt; }
	td.errorlistout		{ border-width: 0; padding: 0; }
	div.errorlistin		{ max-height: 250px; overflow-y: scroll; margin: 0; }
	table.errorlistin	{ width: 100%; border-collapse: collapse; border-width: 0pt; table-layout: fixed; }
	table.errorlistin td
		{ border-width: 1pt 1pt 0 0; vertical-align: top; }
	table.errorlistin tr:first-child td
		{ border-top-width: 0pt; }
	table.errorlistin td.errorsdetail
		{ width: 60%; white-space: nowrap; overflow: hidden; text-overflow: ellipsis; }
	table.errorlistin td.errorsdetail:hover
		{  white-space: normal; }
	table.errorlistin td.errorsdetail:hover a
		{  display: block; }
</style>'''

htmlTableWrap = '<table class="out errorlistout NOPRINT"><tbody><tr><td class="errorlistout"><div class="errorlistin"><table class="errorlistin"><tbody /></table></div></td></tr></tbody></table>'


# 出力シートチェックメソッド
def InsertErrorList(s, e):
  # プラグイン実行イベントデータ
  v = e.Value
  
  # SheetTabInfo ごとにループ
  for sti in v.sheets:
    # HasError 置換ノードを探す
    doc = sti.DocumentTemplateParent
    root = doc.DocumentNode
    cmmErrSpecNode = root.SelectSingleNode("//comment()[contains(.,'{\"spec\":\"HasError\"}') or contains(.,\"{'spec':'HasError'}\")]")
    
    # HasError 置換ノードが見つからなければ、無視
    if cmmErrSpecNode is None:
      continue
    
    # エラー情報が存在しなければ、無視
    if len(sti.Sheet.ExtInfo.Errors) == 0:
      continue
    
    # スタイルノード追加
    nStyle = root.CreateNode(htmlStyleNode)
    nCurHead = root.SelectSingleNode("//head")
    if nCurHead is not None:
      # head 要素の最後尾に設置
      nCurHead.AppendChild(doc.CreateTextNode("\n"))
      nCurHead.AppendChild(nStyle)
      nCurHead.AppendChild(doc.CreateTextNode("\n"))
    else:
      # head が存在しなければ、ルートノードの最後尾に追加
      root.AppendChild(doc.CreateTextNode("\n"))
      root.AppendChild(nStyle)
      root.AppendChild(doc.CreateTextNode("\n"))
    
    
    # テーブルノードをHasError 置換ノードの後ろに追加
    nReturn = doc.CreateTextNode("\n")
    nTableWrap = root.CreateNode(htmlTableWrap)
    cmmErrSpecNode.ParentNode.InsertAfter(nReturn, cmmErrSpecNode)
    cmmErrSpecNode.ParentNode.InsertAfter(nTableWrap, nReturn)
    nTableBody = nTableWrap.SelectSingleNode("tbody/tr/td/div/table/tbody")
    
    # エラーオブジェクトリストごとに、行を追加
    for (key, Title), g in itertools.groupby(sti.Sheet.ExtInfo.Errors, lambda x: (x.ErrTitleKey, x.ErrTitleString)):
      # ErrorItem.ErrTitleKey と .ErrTitleString のタプルを使ってグループ化している
      nFirstTd = None	# 項目名のTd
      lastIndex = 0
      
      for i, erritem in enumerate(g):
        lastIndex = i
        nTr = None
        elmLink = None
        if i == 0:
          # グループ化された1行目
          nTr = root.CreateNode('<tr><td class="errorstitle" /><td class="errorsdetail"><a /></td></tr>')
          nFirstTd = nTr.SelectSingleNode("td[1]")
          nFirstTd.AppendChild(doc.CreateTextNode(erritem.ErrTitleString))
          elmLink = nTr.SelectSingleNode("td[2]/a")
          
        else:
          # グループ化された2行目以降
          nTr = root.CreateNode('<tr><td class="errorsdetail"><a /></td></tr>')
          elmLink = nTr.SelectSingleNode("td[1]/a")
        
        # リンクを追加する
        elmLink.AppendChild(doc.CreateTextNode(erritem.ErrMsgString))
        elmLink.SetAttributeValue('title', erritem.ErrMsgString)	# ツールチップ用
        if erritem.Path is not None:
          elmLink.SetAttributeValue('href', '#' + str(erritem.Path).replace('[', '').replace(']', '') + '.value')
        
        # 列をテーブルに追加する
        nTableBody.AppendChild(nTr)
      
      # rowspan の設定
      nFirstTd.SetAttributeValue('rowspan', str(lastIndex + 1))
    


cse.addhandler("BeforeMakeCheckSheet", InsertErrorList)
