﻿# -*- coding: utf-8 -*-
"""
製品情報ダイアログにドロップされた、CTR/WUP の提出確認書を読み込みます。

MakeTranslateList で、.NET クラスを使用しています。
"""

import sys
import clr
import cse
import re
import datetime

import cseMessageBox as mb
clr.AddReference("System.Xml")
import System.Xml.XmlDocument

# CTR 提出確認書変換
class CtrSbmSheetCnv:
  def __init__(self, xml):
    # チェックシートの言語フォーマットを確認
    isJaLng = (xml.SelectSingleNode("/ctr-master-editor/@language").Value.lower() == 'ja' )
    
    elmsbm = xml.SelectSingleNode("/ctr-master-editor/submission-info")
    elmtitle = elmsbm.SelectSingleNode("product-name")
    elmsbmver = elmsbm.SelectSingleNode("submit-version")
    elmsbmdate = elmsbm.SelectSingleNode("submit-date")
    
    elmcxi = xml.SelectSingleNode("/ctr-master-editor//cxi-list/cxi")
    if elmcxi != None:
      elmprodcode = elmcxi.SelectSingleNode("product-code")
      elmremver = elmcxi.SelectSingleNode("remaster-version")
    else:
      elmprodcode = xml.SelectSingleNode("//product-code")
      elmremver = xml.SelectSingleNode("//data-title-version")
    
    mcode = re.match("(\w{3})-(\w)-(\w{4}(?:-[0-9a-fA-F]{2})?)$", elmprodcode.InnerText)
    mdate = re.match("(\d+)/(\d+)/(\d+)$", elmsbmdate.InnerText)
    
    self.strTitle = elmtitle.InnerText
    self.strClassCode = mcode.group(1)
    self.strFormCode = mcode.group(2)
    self.strInitCode = mcode.group(3)
    self.strRemVer = int(elmremver.InnerText)
    self.strSbmVer = int(elmsbmver.InnerText)
    if isJaLng:
      # 日本版は yyyy/mm/dd 型式
      self.strSbmYear = int(mdate.group(1))
      self.strSbmMonth = int(mdate.group(2))
      self.strSbmDay = int(mdate.group(3))
    else:
      # 英語版は mm/dd/yyyy 型式
      self.strSbmYear = int(mdate.group(3))
      self.strSbmMonth = int(mdate.group(1))
      self.strSbmDay = int(mdate.group(2))
    

# WUP 提出確認書変換
class WupSbmSheetCnv:
  def __init__(self, xml):
    # チェックシートの言語フォーマットを確認
    isJaLng = (xml.SelectSingleNode("/wup-master-info/@language").Value.lower() == 'ja' )
    
    elmsbm = xml.SelectSingleNode("/wup-master-info/submission-info")
    elmtitle = elmsbm.SelectSingleNode("article")
    elmsbmver = elmsbm.SelectSingleNode("submit-version")
    elmsbmdate = elmsbm.SelectSingleNode("submit-date")
    
    elmpart = xml.SelectSingleNode("/wup-master-info/partition-list/partition[@name='p01']")
    elmremver = elmpart.SelectSingleNode("remaster-version")
    
    self.strTitle = elmtitle.InnerText
    self.strClassCode = elmpart.SelectSingleNode("content-platform").InnerText
    self.strFormCode = elmpart.SelectSingleNode("form-code").InnerText
    initcode = elmpart.SelectSingleNode("initial-code").InnerText
    variation = elmpart.SelectSingleNode("variation").InnerText
    if self.strFormCode != "M":
      self.strInitCode = initcode
    else:
      self.strInitCode = initcode + '-' + variation[2:]
    
    self.strRemVer = int(elmremver.InnerText)
    self.strSbmVer = int(elmsbmver.InnerText)
    self.strSbmYear = int(elmsbmdate.SelectSingleNode("submit-year").InnerText)
    self.strSbmMonth = int(elmsbmdate.SelectSingleNode("submit-month").InnerText)
    self.strSbmDay = int(elmsbmdate.SelectSingleNode("submit-day").InnerText)
    

# 提出確認書
def OnDropSubmissionSheet(s, e):
  # エラーの種類
  errmsgfileload = cse.getapplang("PlgErr_ReadSbmt_FileLoading")
  # プラグイン実行イベントデータ
  v = e.Value
  
  if len(v.filenames) <= 0:
    return
  
  # 提出確認書を読み取る
  xml = System.Xml.XmlDocument()
  try:
    xml.Load(v.filenames[0])
    
    if not xml.SelectSingleNode('/ctr-master-editor') is None:
      sho = CtrSbmSheetCnv(xml)
    elif not xml.SelectSingleNode('/wup-master-info') is None:
      sho = WupSbmSheetCnv(xml)
    else:
      raise Exception
    
  except:
    # 例外が発生したら、ダイアログを表示
    try:
      # フォーマット可能なら、フォーマットする
      errmsgfileload = errmsgfileload % v.filenames[0]
    except:
      pass
    mb.showerror("Error", errmsgfileload)
    return
  
  v.prodinfotmp.Title = sho.strTitle
  v.prodinfotmp.ClassCode = sho.strClassCode
  v.prodinfotmp.FormCode = sho.strFormCode
  v.prodinfotmp.InitCode = sho.strInitCode
  v.prodinfotmp.RemVer = sho.strRemVer
  v.prodinfotmp.SbmVer = sho.strSbmVer
  v.prodinfotmp.SbmYear = sho.strSbmYear
  v.prodinfotmp.SbmMonth = sho.strSbmMonth
  v.prodinfotmp.SbmDay = sho.strSbmDay
  

cse.addhandler("ProdSettingFileDrop", OnDropSubmissionSheet)


